@@ -2,7 +2,7 @@
 /*
  * gsf-libxml.h: Utility wrappers for using gsf with libxml
  *
- * Copyright (C) 2002-2005 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2002-2006 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2.1 of the GNU Lesser General Public
@@ -32,7 +32,7 @@ G_BEGIN_DECLS
 /* GSF wrappers for libxml2 */
 xmlParserCtxt *gsf_xml_parser_context (GsfInput   *input);
 #if 0
-				       /* this is cleaner, tack it on for 2.0 */
+					/* this is cleaner, tack it on for 2.0 */
 					xmlSAXHandlerPtr sax, gpointer user);
 #endif
 int	       gsf_xmlDocFormatDump   (GsfOutput  *output,
@@ -41,47 +41,37 @@ int	       gsf_xmlDocFormatDump   (GsfOutput  *output,
 				       gboolean    format);
 
 /****************************************************************************/
-/* Simplified GSF based xml import (based on libxml2 SAX) */
-typedef struct _GsfXMLBlob	GsfXMLBlob;
-
+/* Simplified wrapper to SAX based xml import */
 typedef struct _GsfXMLIn		GsfXMLIn;
 typedef struct _GsfXMLInDoc		GsfXMLInDoc;
-typedef struct _GsfXMLInDocExtension	GsfXMLInDocExtension;
 typedef struct _GsfXMLInNode		GsfXMLInNode;
 typedef struct _GsfXMLInNS		GsfXMLInNS;
-
+typedef struct _GsfXMLBlob		GsfXMLBlob;
 typedef enum {
-	GSF_XML_NO_CONTENT,
+	GSF_XML_NO_CONTENT = FALSE,
 	GSF_XML_CONTENT,
 	GSF_XML_SHARED_CONTENT
 } GsfXMLContent;
+typedef gboolean (*GsfXMLInUnknownFunc) (GsfXMLIn *xin,
+					 xmlChar const *elem, xmlChar const **attrs);
+typedef void     (*GsfXMLInExtDtor)     (GsfXMLIn *xin, gpointer old_state);
 
-typedef gboolean (*GsfXMLInUnknownFunc) (GsfXMLIn *state, xmlChar const *elem, xmlChar const **attrs);
 struct _GsfXMLIn {
-	GsfXMLInDoc  const *doc;	/* init before parsing */
-
-    /* look but do not change */
-	GsfXMLInNode const *node;	/* current node */
-	GSList	 	   *state_stack;
-
-	GsfXMLInNS   const *default_ns;	/* optionally NULL */
-	GSList	 	   *ns_stack;
-
-	GString		*content;
-	gint		 unknown_depth;	/* handle recursive unknown tags */
-	GHashTable	*ns_prefixes;	/* current ns prefixes */
-	GPtrArray	*ns_by_id;		/* indexed by id */
-
-	gpointer	 user_state;
+	/* public state : read only */
+	gpointer	    user_state;
+	GString		   *content;
+	GsfXMLInDoc  const *doc;
+	GsfXMLInNode const *node;	/* current node (not on the stack) */
+	GSList	 	   *node_stack;	/* stack of GsfXMLInNode */
 };
 
 struct _GsfXMLInNode {
-	char const *id;
+	char const *id;		/* unique in the entire tree */
 	int	    ns_id;
 	char const *name;
 	char const *parent_id;
-	void (*start) (GsfXMLIn *state, xmlChar const **attrs);
-	void (*end)   (GsfXMLIn *state, GsfXMLBlob *unknown);
+	void (*start) (GsfXMLIn *xin, xmlChar const **attrs);
+	void (*end)   (GsfXMLIn *xin, GsfXMLBlob *unknown);
 
 	union {
 		int	    v_int;
@@ -93,10 +83,6 @@ struct _GsfXMLInNode {
 
 	unsigned int check_children_for_ns : 1;
 	unsigned int share_children_with_parent : 1;
-
-	/* internal state */
-	unsigned int parent_initialized : 1;
-	GSList *groups;
 };
 
 struct _GsfXMLInNS {
@@ -104,35 +90,39 @@ struct _GsfXMLInNS {
 	unsigned    ns_id;
 };
 
-struct _GsfXMLInDocExtension {
-};
-
 #define GSF_XML_IN_NS(id, uri) \
-{ uri, id}
+{ uri, id }
+#define GSF_XML_IN_NS_END \
+{ NULL, 0 }
 
 #define GSF_XML_IN_NODE_FULL(parent_id, id, ns, name, has_content, 	\
 			     share_children_with_parent, check_ns, start, end, user)	\
 {									\
 	#id, ns, name, #parent_id, start, end, { user }, has_content,	\
 	check_ns, share_children_with_parent,				\
-	FALSE, NULL							\
 }
 
 #define GSF_XML_IN_NODE(parent_id, id, ns, name, has_content, start, end) \
 	GSF_XML_IN_NODE_FULL(parent_id, id, ns, name, has_content,	  \
 			     FALSE, FALSE, start, end, 0)
+#define GSF_XML_IN_NODE_END	\
+	{ NULL, 0, NULL, NULL, NULL, NULL, { 0 }, GSF_XML_NO_CONTENT, FALSE, FALSE }
 
-GsfXMLInDoc *gsf_xml_in_doc_new	   (GsfXMLInNode *root, GsfXMLInNS *ns);
+GsfXMLInDoc *gsf_xml_in_doc_new	   (GsfXMLInNode const *nodes, GsfXMLInNS const *ns);
 void	     gsf_xml_in_doc_free   (GsfXMLInDoc *doc);
 gboolean     gsf_xml_in_doc_parse  (GsfXMLInDoc *doc, GsfInput *input,
 				    gpointer user_state);
-void	     gsf_xml_in_doc_extend (GsfXMLInDoc *doc, GsfXMLInNode *nodes);
 void	     gsf_xml_in_doc_set_unknown_handler (GsfXMLInDoc *doc,
-				    GsfXMLInUnknownFunc handler);
+						 GsfXMLInUnknownFunc handler);
 
-char const  *gsf_xml_in_check_ns   (GsfXMLIn const *state, char const *str,
+void	     gsf_xml_in_push_state (GsfXMLIn *xin, GsfXMLInDoc const *doc,
+				    gpointer new_state, GsfXMLInExtDtor dtor,
+				    xmlChar const **attrs);
+
+GsfInput    *gsf_xml_in_get_input  (GsfXMLIn const *xin);
+char const  *gsf_xml_in_check_ns   (GsfXMLIn const *xin, char const *str,
 				    unsigned int ns_id);
-gboolean     gsf_xml_in_namecmp	   (GsfXMLIn const *state, char const *str,
+gboolean     gsf_xml_in_namecmp	   (GsfXMLIn const *xin, char const *str,
 				    unsigned int ns_id, char const *name);
 
 /****************************************************************************/
@@ -144,42 +134,47 @@ typedef struct _GsfXMLOut	GsfXMLOut;
 #define GSF_XML_OUT(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), GSF_XML_OUT_TYPE, GsfXMLOut))
 #define GSF_IS_XML_OUT(o)	(G_TYPE_CHECK_INSTANCE_TYPE ((o), GSF_XML_OUT_TYPE))
 
-GType gsf_xml_out_get_type (void);
+GType gsf_xml_out_get_type      (void) G_GNUC_CONST;
+/* void  gsf_xml_out_register_type (GTypeModule *module); glib dynamic types are not thread safe */
+
 GsfXMLOut *gsf_xml_out_new (GsfOutput *output);
 
-void	    gsf_xml_out_set_doc_type	(GsfXMLOut *xml, char const *type);
-void	    gsf_xml_out_start_element	(GsfXMLOut *xml, char const *id);
-char const *gsf_xml_out_end_element	(GsfXMLOut *xml);
+void	    gsf_xml_out_set_doc_type	(GsfXMLOut *xout, char const *type);
+void	    gsf_xml_out_start_element	(GsfXMLOut *xout, char const *id);
+char const *gsf_xml_out_end_element	(GsfXMLOut *xout);
 
-void gsf_xml_out_simple_element		(GsfXMLOut *xml, char const *id,
+void gsf_xml_out_simple_element		(GsfXMLOut *xout, char const *id,
 					 char const *content);
-void gsf_xml_out_simple_int_element	(GsfXMLOut *xml, char const *id,
+void gsf_xml_out_simple_int_element	(GsfXMLOut *xout, char const *id,
 					 int val);
-void gsf_xml_out_simple_float_element	(GsfXMLOut *xml, char const *id,
+void gsf_xml_out_simple_float_element	(GsfXMLOut *xout, char const *id,
 					 double val, int precision);
 
-void gsf_xml_out_add_cstr_unchecked	(GsfXMLOut *xml, char const *id,
+void gsf_xml_out_add_cstr_unchecked	(GsfXMLOut *xout, char const *id,
 					 char const *val_utf8);
-void gsf_xml_out_add_cstr		(GsfXMLOut *xml, char const *id,
+void gsf_xml_out_add_cstr		(GsfXMLOut *xout, char const *id,
 					 char const *val_utf8);
-void gsf_xml_out_add_bool		(GsfXMLOut *xml, char const *id,
+void gsf_xml_out_add_bool		(GsfXMLOut *xout, char const *id,
 					 gboolean val);
-void gsf_xml_out_add_int		(GsfXMLOut *xml, char const *id,
+void gsf_xml_out_add_int		(GsfXMLOut *xout, char const *id,
 					 int val);
-void gsf_xml_out_add_uint		(GsfXMLOut *xml, char const *id,
+void gsf_xml_out_add_uint		(GsfXMLOut *xout, char const *id,
 					 unsigned int val);
-void gsf_xml_out_add_float		(GsfXMLOut *xml, char const *id,
+void gsf_xml_out_add_float		(GsfXMLOut *xout, char const *id,
 					 double val, int precision);
-void gsf_xml_out_add_color		(GsfXMLOut *xml, char const *id,
+void gsf_xml_out_add_color		(GsfXMLOut *xout, char const *id,
 					 unsigned int r, unsigned int g, unsigned int b);
-void gsf_xml_out_add_base64		(GsfXMLOut *xml, char const *id,
+void gsf_xml_out_add_base64		(GsfXMLOut *xout, char const *id,
 					 guint8 const *data, unsigned int len);
-void gsf_xml_out_add_enum               (GsfXMLOut *xml, char const *id,
+void gsf_xml_out_add_enum               (GsfXMLOut *xout, char const *id,
 					 GType etype, gint val);
+void gsf_xml_out_add_gvalue             (GsfXMLOut *xout, char const *id,
+					 GValue const *val);
 
 /****************************************************************************/
 /* Some general utilities */
 gboolean gsf_xml_gvalue_from_str (GValue *res, GType t, char const *str);
+GsfOutput *gsf_xml_out_get_output (GsfXMLOut const *xout);
 
 G_END_DECLS
 
